#!/usr/bin/perl -- -*-Perl-*-

# The purpose of this script is to repost alt.ql.creative stories
# on both alt.ql.creative and alt.tv.quantum-leap.creative.
# Note: It might be nice to repost to alt.drwho.creative
# or alt.tv.x-files or other *.creative newsgroups if possible...

# We might be in the directory to repost if we were given no arguments

$debug = 0;
$verbose = 0;
$cc_user = '';
$standard_user = 'ql-archive@NOSPAMemployeesPLEASE.org';
$user = $standard_user;
$aqcdbm = $ENV{'HOME'}.'.aqc-repost';

if (scalar(@ARGV) == 0) {
    chop($dir = `pwd`);
    unshift(@ARGV, $dir);
}

while ($ARGV[0] =~ /^-/) {
    if ($ARGV[0] eq "-d") {
	$debug++;		# turn on debugging
	shift;
    }
    elsif ($ARGV[0] eq "-v") {
	$verbose = 1;
	shift;
    }
    elsif ($ARGV[0] eq "-cc-user") {
	$cc_user = shift;
	shift;
    }
    elsif ($ARGV[0] eq "-u") {
	shift;
	$user = shift;
	print "From: $user\n" if ($debug);
    }
    else {
	print STDERR "Usage: $0 [-v] [-d] [-u user] directory...\n";
	exit(1);
    }
}

foreach $dir (@ARGV) {

    chop($dir) while ($dir =~ m,/$,);
    ($pdir = $dir) =~ s,[^/]*/,,g;
    print "pdir is $pdir\n" if ($verbose);

    if (! -d $dir) {
	if (-d "/users/ftp/mdb/ql-archive/alt.ql.creative/$dir") {
	    $dir = "/users/ftp/mdb/ql-archive/alt.ql.creative/$dir";
	}
	else {
	    print "\n$pdir is not the name of a story directory\n";
	    next;
	}
    }

    opendir(DIR, $dir) || die "unable to open $dir: $!";
    @files = ();
    while($file = readdir(DIR)) {
	next if ($file eq '.ls_allowed');
	next if ( -d $file);
	push(@files, $file);
    }
    closedir(DIR);

    @guides = grep(/guide$/, @files);
    @parts  = sort(grep(/\.p\d+$/, @files));

    $xguide = 0;
    $title = '';
    $author = '';
    $guide = '';
    $sge_before = ' ' x 15 . '=' x 7 . ' ';
    $sge_after = '=' x 7 . "\n";
    $sge = ' Story Guide Entry ';
    $guide_start = $sge_before . 'start' . $sge . $sge_after;
    $guide_end   = "\n " . $sge_before . 'end' . $sge . $sge_after ."\n\n";
    foreach $file (@guides) {
	print "READING $dir/$file ...\n" if ($debug);
	@guide_text = &read_file("$dir/$file");
	($nop, $title) = split(': ', (grep(/TITLE:/,@guide_text))[0], 2);
	($nop, $author) = split(': ',(grep(/AUTHOR:/,@guide_text))[0], 2);
	if ($title eq '') {
	    die "problem reading title from $dir/$file\n";
	}
	print "title: $title\n" if ($verbose);
	if ($author eq '') {
	    die "problem reading author from $dir/$file\n";
	}
	print "author: $author\n" if ($verbose);
	$guide = join("\n", $guide_start, @guide_text, $guide_end);
	$xtitle{$file}    = $title;
	$xauthor{$file}   = $author;
	$xguidetxt{$file} = $guide;
    }
    if (scalar(@guides) > 1) {
	print("too many guide files found:\n",
	      join("\n",@guides), "\n") if ($verbose); 
	$title  = '';
	$author = '';
	$guide  = '';
	$xguide = 1;
    }

    ($firstp = $parts[$[]) =~ s/^.*\.p//;
    ($lastp  = $parts[$#parts]) =~ s/^.*\.p//;
    $lastpart = sprintf("%-2.2d", scalar(@parts));
    print "First part is $firstp (1)\n" if ($verbose);
    print "Last part is $lastp ($lastpart)\n" if ($verbose);
    $partnum = 0;
    foreach $file (@parts) {
	if (dbmopen(DBM, $aqcdbm, 0644)) {
	    local($val,$oneweek);
	    $oneweek = 60*60*24*7;
	    if (defined($DBM{$file})) {
		$val = $DBM{$file};
		$val = unpack('L',$val);
		if (($val > 0) && (($val + $oneweek) > time)) {
		    print &lasttime("$pdir/$file",$val);
		    next;
		}
	    }
	    dbmclose(DBM);
	}

	($partnum = $file) =~ s/^.*\.p//;
	if ($xguide) {
	    $title  = $xtitle{$file.'guide'};
	    $author = $xauthor{$file.'guide'};
	    $guide  = $xguidetxt{$file.'guide'};
	}
	$subject = sprintf("REPOST: $title, Part%-2.2d/%-2.2d",
			   $partnum, $lastp);
	@text = &read_file("$dir/$file");
	@header = ();
	while (scalar(@text) && ($text[$[] ne '')) {
	    $line = shift(@text);
	    if ($line =~ /^(From:|Subject:|Date:)/) {
		push(@header, $line);
	    }
	}
	# remove all trailing empty lines
	while ($text[$#text] =~ /^\s*$/) {
	    pop(@text);
	}
	push(@text, '');	# add one trailing empty line back

	# make sure we don't send a sendmail termination character
	# add a newline at the end of each line
	foreach $line (@text) {
	    $line = '..' if ($line eq '.');
	    $line .= "\n";
	}

	if ($title eq '') {
	    print "Problem with title... skipping\n";
	    next;
	}
	if ($debug > 1) {
	    print "Subject: $subject\n";
	    next;
	}
	&post_message($user, $cc_user, $pdir, $file, $subject,
		      join("\n", @header)."\n", $guide, @text);
	$guide = '';
    }
}
exit 0;

sub post_message {
    local($user,$cc_user,$pdir,$file,$subject,$header,$guide,@text) = @_;
    local($sendto,$descr);
    local($cmd) = '/usr/lib/sendmail';
    local(@header);
    local(@addrs);

    $cmd = '/users/ftp/mdb/ql-archive/programs/cmdecho' if ($debug);

    if ( -x $cmd ) {
	@addrs = ("alt\#ql\#creative\@newsgate\.cisco\.com",
		  "alt\#tv\#quantum-leap\#creative\@newsgate\.cisco\.com"
		  );
	if (($cc_user ne '') && ($cc_user ne $standard_user)) {
	    push(@addrs, $cc_user);
	}
	push(@addrs, $standard_user) if ($debug);

	@header = ("From: $standard_user (Mark D. Baushke)\n",
		   "Subject: $subject\n",
		   "Precedence: bulk\n",
		   "Organization: Quantum Leap\n\n");
	if ($user ne $standard_user) {
	    push(@header, "Repost-Requested-by: $user\n");
	}
	push(@header,
	     "Submitted-by: $author\n",
	     "Archive-URL:  ftp://ftp.employees.org/mdb/ql-archive/alt.ql.creative/\n",
	     "Archive-name: $pdir/$file\n\n\n");
	push(@header, $guide) if ($guide ne '');
	push(@header, $header) if ($header ne '');

 	foreach $sendto (@addrs) {
	    ($descr = $sendto) =~ s,\@.*$,,;
	    $descr =~ s,\#,.,g;
	    if (&send_mail("|$cmd -f $user $sendto", 
			   "To: $standard_user ($descr)\n",
			   @header, @text)) {
		if ($user eq $standard_user) {
		    print "Sent article w/Subject: $subject\n";
		    print "(Sent part $partnum to \'$sendto\'.)\n\n";
		}
		else {
		    print "Sent article w/Subject: $subject\n";
		}
		sleep(10);

		if ($debug == 0) {
		    if (dbmopen(DBM, $aqcdbm, 0644)) {
			local($key) = $file;
			$DBM{$key} = pack('L',time);
			dbmclose(DBM);
		    }
		}
	    }
	}
    }
}

sub send_mail {
    local($sendcmd, @text) = @_;

    if (open(MAIL, $sendcmd)) {
	print MAIL @text;
	close(MAIL);
	return 1;
    }
    0;
}

sub read_file {
    local(@text);
    local($filename) = @_;

    open(FILE, "<$filename") || return ("OOPS: reading $filename: $!");
    while (<FILE>) {
	chop;
	push(@text, $_);
    }
    close(FILE);
    @text;
}

sub lasttime {
    local($file,$time) = @_;
    local(@Months) = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
	   'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
    local(@Days) = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');

    ($sec,$min,$hour,$mday,$mon,$year,$wday) = gmtime($time);
    $year += 1900;
    $text = sprintf("%s, %2d %s %4d %-2.2d:%-2.2d:%-2.2d UTC reposted: %s\n",
		    $Days[$wday], $mday, $Months[$mon], $year, $hour, $min,
		    $sec, $file);
}
